# -*- coding: UTF-8 -*-

'''
Module
    __init__.py
Copyright
    Copyright (C) 2018 - 2026 Vladimir Roncevic <elektron.ronca@gmail.com>
    gen_avr8 is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    gen_avr8 is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
Info
    Defines class AVR8Setup with attribute(s) and method(s).
    Generates AVR project skeleton.
'''

import sys
from typing import Any, Dict, List, Optional

try:
    from ats_utilities.checker import ATSChecker
    from ats_utilities.console_io.verbose import verbose_message
    from ats_utilities.config_io.yaml.yaml2object import Yaml2Object
    from ats_utilities.exceptions.ats_type_error import ATSTypeError
    from gen_avr8.pro.module_type import ModuleType
    from gen_avr8.pro.mcu_selector import MCUSelector
    from gen_avr8.pro.osc_selector import OSCSelector
    from gen_avr8.pro.template_dir import TemplateDir
    from gen_avr8.pro.template_type import TemplateType
    from gen_avr8.pro.read_template import ReadTemplate
    from gen_avr8.pro.write_template import WriteTemplate
except ImportError as ats_error_message:  # pragma: no cover
    # Force exit python #######################################################
    sys.exit(f'\n{__file__}\n{ats_error_message}\n')  # pragma: no cover

__author__: str = 'Vladimir Roncevic'
__copyright__: str = '(C) 2026, https://vroncevic.github.io/gen_avr8'
__credits__: List[str] = ['Vladimir Roncevic', 'Python Software Foundation']
__license__: str = 'https://github.com/vroncevic/gen_avr8/blob/dev/LICENSE'
__version__: str = '2.6.3'
__maintainer__: str = 'Vladimir Roncevic'
__email__: str = 'elektron.ronca@gmail.com'
__status__: str = 'Updated'


class AVR8Setup(ATSChecker):
    '''
        Defines class AVR8Setup with attribute(s) and method(s).
        Generates AVR project skeleton.

        It defines:

            :attributes:
                | _GEN_VERBOSE - Console text indicator for process-phase.
                | _mcu_sel - MCU selector API.
                | _fosc_sel - FOSC selector API.
                | _reader - Reader API.
                | _writer - Writer API.
                | _setup - Project setup.
            :methods:
                | __init__ - Initials AVR8Setup constructor.
                | project_setup - Property methods for set operations.
                | gen_pro_setup - Generates project skeleton.
    '''

    _GEN_VERBOSE: str = 'GEN_AVR8::PRO::AVR8SETUP'

    def __init__(self, verbose: bool = False) -> None:
        '''
            Initials AVR8Setup constructor.

            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :exceptions: None
        '''
        super().__init__()
        verbose_message(verbose, [f'{self._GEN_VERBOSE.lower()} init setup'])
        self._mcu_sel = MCUSelector(verbose)
        self._fosc_sel = OSCSelector(verbose)
        self._reader = ReadTemplate(verbose)
        self._writer = WriteTemplate(verbose)
        self._setup: Dict[Any, Any] = {}

    def project_setup(
        self,
        project_name: Optional[str],
        project_type: Optional[str],
        verbose: bool = False
    ) -> None:
        '''
            Property methods for set operations.

            :param project_name: Project name | None
            :type project_name: <Optional[str]>
            :param project_type: Project type | None
            :type project_type: <Optional[str]>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :exceptions: ATSTypeError
        '''
        error_msg: Optional[str] = None
        error_id: Optional[int] = None
        error_msg, error_id = self.check_params([
            ('str:project_name', project_name),
            ('str:project_type', project_type)
        ])
        if error_id == self.TYPE_ERROR:
            raise ATSTypeError(error_msg)
        verbose_message(
            verbose,
            [f'{self._GEN_VERBOSE.lower()} set {project_name} {project_type}']
        )
        self._setup.update({'name': project_name})
        self._setup.update({'type': project_type})

    def gen_pro_setup(self, verbose: bool = False) -> bool:
        '''
            Generates AVR8 project setup.

            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :return: True (success operation) | False
            :rtype: <bool>
            :exceptions: None
        '''
        status: bool = False
        if any(['name' not in self._setup, 'type' not in self._setup]):
            return status
        if any([
            not bool(self._setup['name']), not bool(self._setup['type'])
        ]):
            return status
        verbose_message(
            verbose, [
                f'{self._GEN_VERBOSE.lower()} gen project',
                self._setup['type'], self._setup['name']
            ]
        )
        conf_dir: Optional[str] = TemplateDir.setup_conf_dir(verbose)
        if not bool(conf_dir):
            return status
        setup_template: Optional[str] = TemplateType.setup_template_type(
            self._setup['type'], verbose
        )
        if not bool(setup_template):
            return status
        yml2obj: Yaml2Object = Yaml2Object(f'{conf_dir}{setup_template}')
        all_stat: List[bool] = []
        if bool(yml2obj):
            pro_cfg: Dict[str, str] | None = yml2obj.read_configuration()
            pro_data: Dict[Any, Any] = {}
            if bool(pro_cfg):
                templates: List[str] | None = pro_cfg['templates'].split(' ')
                modules: List[str] | None = pro_cfg['modules'].split(' ')
                if any([not bool(templates), not bool(modules)]):
                    return status
                pro_data['name'] = self._setup['name']
                pro_data['type'] = self._setup['type']
                pro_data['mcu'] = self._mcu_sel.choose_mcu(verbose)
                pro_data['osc'] = self._fosc_sel.choose_osc(verbose)
                if any([not bool(pro_data['mcu']), not bool(pro_data['osc'])]):
                    return status
                self._writer.pro_dir = self._setup['name']
                for template, module in zip(templates, modules):
                    template_dir: Optional[
                        str
                    ] = TemplateDir.setup_template_dir(verbose)
                    pro_data['template'] = self._reader.read(
                        f'{template_dir}{self._setup["type"]}/{template}',
                        verbose
                    )
                    pro_data['module'] = ModuleType.pre_process_module(
                        self._setup['type'],
                        self._setup['name'],
                        module, verbose
                    )
                    all_stat.append(self._writer.write(pro_data, verbose))
                if all(all_stat) and len(templates) == len(all_stat):
                    status = True
        return status



# -*- coding: UTF-8 -*-

'''
Module
    read_template.py
Copyright
    Copyright (C) 2018 - 2026 Vladimir Roncevic <elektron.ronca@gmail.com>
    gen_avr8 is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    gen_avr8 is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
Info
    Defines class ReadTemplate with attribute(s) and method(s).
    Creates an API for reading a template files.
'''

import sys
from typing import List, Optional

try:
    from ats_utilities.config_io.file_check import FileCheck
    from ats_utilities.console_io.verbose import verbose_message
    from ats_utilities.exceptions.ats_type_error import ATSTypeError
except ImportError as ats_error_message:  # pragma: no cover
    # Force exit python #######################################################
    sys.exit(f'\n{__file__}\n{ats_error_message}\n')  # pragma: no cover

__author__: str = 'Vladimir Roncevic'
__copyright__: str = '(C) 2026, https://vroncevic.github.io/gen_avr8'
__credits__: List[str] = ['Vladimir Roncevic', 'Python Software Foundation']
__license__: str = 'https://github.com/vroncevic/gen_avr8/blob/dev/LICENSE'
__version__: str = '2.6.3'
__maintainer__: str = 'Vladimir Roncevic'
__email__: str = 'elektron.ronca@gmail.com'
__status__: str = 'Updated'


class ReadTemplate(FileCheck):
    '''
        Defines class ReadTemplate with attribute(s) and method(s).
        Creates an API for reading a template files.

        It defines:

            :attributes:
                | _GEN_VERBOSE - Console text indicator for process-phase.
                | _FORMAT - File format for template.
            :methods:
                | __init__ - Initials ReadTemplate constructor.
                | read - Reads template file.
    '''

    _GEN_VERBOSE: str = 'GEN_AVR8::PRO::READ_TEMPLATE'
    _FORMAT: str = 'template'

    def __init__(self, verbose: bool = False) -> None:
        '''
            Initials ReadTemplate constructor.

            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :exceptions: None
        '''
        super().__init__(verbose)
        verbose_message(verbose, [f'{self._GEN_VERBOSE.lower()} init reader'])

    def read(
        self, template_file: Optional[str], verbose: bool = False
    ) -> Optional[str]:
        '''
            Reads template file.

            :param template_file: Template file path | None
            :type template_file: <Optional[str]>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :return: Template content for module | None
            :rtype: <Optional[str]>
            :exceptions: ATSTypeError
        '''
        error_msg: Optional[str] = None
        error_id: Optional[int] = None
        error_msg, error_id = self.check_params([
            ('str:template_file', template_file)
        ])
        if error_id == self.TYPE_ERROR:
            raise ATSTypeError(error_msg)
        setup_content: Optional[str] = None
        self.check_path(template_file, verbose)
        self.check_mode('r', verbose)
        self.check_format(template_file, self._FORMAT, verbose)
        if bool(template_file) and self.is_file_ok():
            verbose_message(
                verbose, [f'{self._GEN_VERBOSE.lower()} load template']
            )
            with open(template_file, 'r', encoding='utf-8') as template:
                if bool(template):
                    setup_content = template.read()
        return setup_content



# -*- coding: UTF-8 -*-

'''
Module
    osc_selector.py
Copyright
    Copyright (C) 2018 - 2026 Vladimir Roncevic <elektron.ronca@gmail.com>
    gen_avr8 is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    gen_avr8 is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
Info
    Defines class OSCSelector with attribute(s) and method(s).
    Selects FOSC for generation process of project structure.
'''

import sys
from typing import Dict, List, Optional
from os.path import dirname, realpath

try:
    from ats_utilities.console_io.error import error_message
    from ats_utilities.config_io.file_check import FileCheck
    from ats_utilities.console_io.verbose import verbose_message
    from ats_utilities.config_io.yaml.yaml2object import Yaml2Object
except ImportError as ats_error_message:  # pragma: no cover
    # Force exit python #######################################################
    sys.exit(f'\n{__file__}\n{ats_error_message}\n')  # pragma: no cover

__author__: str = 'Vladimir Roncevic'
__copyright__: str = '(C) 2026, https://vroncevic.github.io/gen_avr8'
__credits__: List[str] = ['Vladimir Roncevic', 'Python Software Foundation']
__license__: str = 'https://github.com/vroncevic/gen_avr8/blob/dev/LICENSE'
__version__: str = '2.6.3'
__maintainer__: str = 'Vladimir Roncevic'
__email__: str = 'elektron.ronca@gmail.com'
__status__: str = 'Updated'


class OSCSelector(FileCheck):
    '''
        Defines class OSCSelector with attribute(s) and method(s).
        Selects FOSC for generation process of project structure.

        It defines:

            :attributes:
                | _GEN_VERBOSE - Console text indicator for process-phase.
                | _FOSC_LIST - Configuration file with FOSC list.
                | _fosc_list - FOSC list.
            :methods:
                | __init__ - Initials OSCSelector constructor.
                | get_fosc_list - Gets for FOSC list object.
                | choose_osc - Selects FOSC for target.
    '''

    _GEN_VERBOSE: str = 'GEN_AVR8::PRO::OSC_SELECTOR'
    _FOSC_LIST: str = '/../conf/fosc.yaml'

    def __init__(self, verbose: bool = False) -> None:
        '''
            Initials OSCSelector constructor.

            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :exceptions: None
        '''
        super().__init__(verbose)
        verbose_message(
            verbose, [f'{self._GEN_VERBOSE.lower()} init fosc selector']
        )
        fosc_list: str = f'{dirname(realpath(__file__))}{self._FOSC_LIST}'
        self.check_path(fosc_list, verbose)
        self.check_mode('r', verbose)
        self.check_format(fosc_list, 'yaml', verbose)
        self._fosc_list: Optional[List[str]] = None
        if self.is_file_ok():
            yml2obj: Optional[Yaml2Object] = Yaml2Object(fosc_list)
            fosc_cfg: Dict[str, str] | None = yml2obj.read_configuration()
            if bool(fosc_cfg) and 'osc' in fosc_cfg:
                self._fosc_list = fosc_cfg['osc'].split(' ')

    def get_fosc_list(self) -> Optional[List[str]]:
        '''
            Gets for FOSC list object.

            :return: FOSC configuration | None
            :rtype: <Optional[List[str]]>
            :exceptions: None
        '''
        return self._fosc_list

    def choose_osc(self, verbose: bool = False) -> Optional[str]:
        '''
            Select FOSC for target.

            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :return: FOSC | None
            :rtype: <Optional[str]>
            :exceptions: None
        '''
        verbose_message(verbose, [f'{self._GEN_VERBOSE.lower()} select FOSC'])
        fosc_name_index: int = -1
        fosc_name: Optional[str] = None
        if self._fosc_list:
            while True:
                print(f'{"#" * 30}\n')
                for index, item in enumerate(self._fosc_list):
                    print(f'\t{index}: {item}')
                print(f'{"#" * 30}\n')
                fosc_name_index = int(input(' select FOSC: '))
                if fosc_name_index not in range(len(self._fosc_list)):
                    error_message(
                        [f'{self._GEN_VERBOSE.lower()} not an option']
                    )
                    continue
                fosc_name = self._fosc_list[fosc_name_index]
                break
        return fosc_name



# -*- coding: UTF-8 -*-

'''
Module
    template_type.py
Copyright
    Copyright (C) 2018 - 2026 Vladimir Roncevic <elektron.ronca@gmail.com>
    gen_avr8 is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    gen_avr8 is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
Info
    Defines class TemplateType with attribute(s) and method(s).
    Creates an API for checking template type and template structure.
'''

import sys
from typing import Dict, List, Optional

try:
    from ats_utilities.checker import ATSChecker
    from ats_utilities.console_io.verbose import verbose_message
    from ats_utilities.exceptions.ats_type_error import ATSTypeError
    from ats_utilities.exceptions.ats_value_error import ATSValueError
except ImportError as ats_error_message:  # pragma: no cover
    # Force exit python #######################################################
    sys.exit(f'\n{__file__}\n{ats_error_message}\n')  # pragma: no cover

__author__: str = 'Vladimir Roncevic'
__copyright__: str = '(C) 2026, https://vroncevic.github.io/gen_avr8'
__credits__: List[str] = ['Vladimir Roncevic', 'Python Software Foundation']
__license__: str = 'https://github.com/vroncevic/gen_avr8/blob/dev/LICENSE'
__version__: str = '2.6.3'
__maintainer__: str = 'Vladimir Roncevic'
__email__: str = 'elektron.ronca@gmail.com'
__status__: str = 'Updated'


class TemplateType:
    '''
        Defines class TemplateType with attribute(s) and method(s).
        Creates an API for checking template type and template structure.

        It defines:

            :attributes:
                | _GEN_VERBOSE - Console text indicator for process-phase.
                | _APP_TEMPLATE - Application type of project.
                | _LIB_TEMPLATE - Library type of project.
                | _TEMPLATE_TYPE - Project template structures.
            :methods:
                | check_template_type - Checks project template type.
                | setup_template_type - Sets template type (app | lib).
    '''

    _GEN_VERBOSE: str = 'GEN_AVR8::PRO::TEMPLATE_TYPE'
    _APP_TEMPLATE: str = 'app'
    _LIB_TEMPLATE: str = 'lib'
    _TEMPLATE_TYPE: Dict[str, str] = {
        _APP_TEMPLATE: 'project_app.yaml',
        _LIB_TEMPLATE: 'project_lib.yaml'
    }

    @classmethod
    def check_template_type(
        cls, template_type: Optional[str], verbose: bool = False
    ) -> bool:
        '''
            Checks project template type (App | Lib).

            :param template_type: Project template type | None
            :type template_type: <Optional[str]>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :return: True (project type for ok) | False
            :rtype: <bool>
            :exceptions: ATSTypeError
        '''
        checker: ATSChecker = ATSChecker()
        error_msg: Optional[str] = None
        error_id: Optional[int] = None
        error_msg, error_id = checker.check_params([
            ('str:template_type', template_type)
        ])
        if error_id == checker.TYPE_ERROR:
            raise ATSTypeError(error_msg)
        verbose_message(
            verbose,
            [f'{cls._GEN_VERBOSE.lower()} checking project ', template_type]
        )
        return template_type in cls._TEMPLATE_TYPE

    @classmethod
    def setup_template_type(
        cls, template_type: str, verbose: bool = False
    ) -> Optional[str]:
        '''
            Sets template type (App | Lib).

            :param template_type: Project template type
            :type template_type: <str>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :return: Project type (app | lib) | None
            :rtype: <Optional[str]>
            :exceptions: ATSTypeError | ATSValueError
        '''
        checker: ATSChecker = ATSChecker()
        error_msg: Optional[str] = None
        error_id: Optional[int] = None
        error_msg, error_id = checker.check_params([
            ('str:template_type', template_type)
        ])
        if error_id == checker.TYPE_ERROR:
            raise ATSTypeError(error_msg)
        if not bool(template_type):
            raise ATSValueError('missing template type')
        if all([
            bool(template_type),
            cls.check_template_type(template_type, verbose)
        ]):
            return cls._TEMPLATE_TYPE[template_type]
        return None



# -*- coding: UTF-8 -*-

'''
Module
    module_type.py
Copyright
    Copyright (C) 2018 - 2026 Vladimir Roncevic <elektron.ronca@gmail.com>
    gen_avr8 is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    gen_avr8 is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
Info
    Defines class ModuleType with attribute(s) and method(s).
    Checks module type (it can be source module | build module).
'''

import sys
from typing import List, Optional

try:
    from ats_utilities.console_io.verbose import verbose_message
except ImportError as ats_error_message:  # pragma: no cover
    # Force exit python #######################################################
    sys.exit(f'\n{__file__}\n{ats_error_message}\n')  # pragma: no cover

__author__: str = 'Vladimir Roncevic'
__copyright__: str = '(C) 2026, https://vroncevic.github.io/gen_avr8'
__credits__: List[str] = ['Vladimir Roncevic', 'Python Software Foundation']
__license__: str = 'https://github.com/vroncevic/gen_avr8/blob/dev/LICENSE'
__version__: str = '2.6.3'
__maintainer__: str = 'Vladimir Roncevic'
__email__: str = 'elektron.ronca@gmail.com'
__status__: str = 'Updated'


class ModuleType:
    '''
        Defines class ModuleType with attribute(s) and method(s).
        Checks module type (it can be source module | build module).

        It defines:

            :attributes:
                | _GEN_VERBOSE - Console text indicator for process-phase.
                | _SOURCE - List of expected source extensions.
                | _BUILD - List of expected build extensions/files.
            :methods:
                | pre_process_module - Processes module name.
                | is_source_module - Checks is source module.
                | is_build_module - Checks is build module.
    '''

    _GEN_VERBOSE: str = 'GEN_AVR8::PRO::MODULE_TYPE'
    _SOURCE: List[str] = ['.c', '.h']
    _BUILD: List[str] = ['.mk', 'Makefile']

    @classmethod
    def pre_process_module(
        cls,
        pro_type: Optional[str],
        pro_name: Optional[str],
        module: Optional[str],
        verbose: bool = False
    ) -> Optional[str]:
        '''
            Processes module name.

            :param pro_type: Project type | None
            :type pro_type: <Optional[str]>
            :param pro_name: Project name | None
            :type pro_name: <Optional[str]>
            :param module: Module name | None
            :type module: <Optional[str]>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :return: Processed module name | None
            :rtype: <Optional[str]>
            :exceptions: None
        '''
        module_name: Optional[str] = None
        if bool(module) and cls.is_source_module(module):
            if pro_type == 'lib':
                if module.endswith(cls._SOURCE[0]):
                    module_name = f'{pro_name}.c'
                if module.endswith(cls._SOURCE[1]):
                    module_name = f'{pro_name}.h'
            elif pro_type == 'app':
                module_name = module
        elif bool(module) and cls.is_build_module(module):
            module_name = module
        verbose_message(
            verbose, [f'{cls._GEN_VERBOSE.lower()} module type', module]
        )
        return module_name

    @classmethod
    def is_source_module(cls, module: Optional[str]) -> bool:
        '''
            Checks is source module.

            :param module: module name | None
            :type module: <Optional[str]>
            :return: True (module is source type) | False
            :rtype: <bool>
            :exceptions: None
        '''
        source_type_supported: bool = False
        if bool(module):
            for source_type_ext in cls._SOURCE:
                if module.endswith(source_type_ext):
                    source_type_supported = True
        return source_type_supported

    @classmethod
    def is_build_module(cls, module: Optional[str]) -> bool:
        '''
            Checks is build module.

            :param module: Module name | None
            :type module: <Optional[str]>
            :return: True (module is build type) | False
            :rtype: <bool>
            :exceptions: None
        '''
        build_type_supported = False
        if bool(module):
            for build_type_ext in cls._BUILD:
                if module.endswith(build_type_ext):
                    build_type_supported = True
        return build_type_supported



# -*- coding: UTF-8 -*-

'''
Module
    write_template.py
Copyright
    Copyright (C) 2018 - 2026 Vladimir Roncevic <elektron.ronca@gmail.com>
    gen_avr8 is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    gen_avr8 is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
Info
    Defines class WriteTemplate with attribute(s) and method(s).
    Creates an API for writing template content with parameters.
'''

import sys
from typing import Dict, List, Optional
from os import getcwd, chmod, makedirs
from datetime import datetime
from string import Template

try:
    from ats_utilities.config_io.file_check import FileCheck
    from ats_utilities.console_io.verbose import verbose_message
    from ats_utilities.exceptions.ats_type_error import ATSTypeError
    from gen_avr8.pro.module_type import ModuleType
except ImportError as ats_error_message:  # pragma: no cover
    # Force exit python #######################################################
    sys.exit(f'\n{__file__}\n{ats_error_message}\n')  # pragma: no cover

__author__: str = 'Vladimir Roncevic'
__copyright__: str = '(C) 2026, https://vroncevic.github.io/gen_avr8'
__credits__: List[str] = ['Vladimir Roncevic', 'Python Software Foundation']
__license__: str = 'https://github.com/vroncevic/gen_avr8/blob/dev/LICENSE'
__version__: str = '2.6.3'
__maintainer__: str = 'Vladimir Roncevic'
__email__: str = 'elektron.ronca@gmail.com'
__status__: str = 'Updated'


class WriteTemplate(FileCheck):
    '''
        Defines class WriteTemplate with attribute(s) and method(s).
        Creates an API for writing template content with parameters.

        It defines:

            :attributes:
                | _GEN_VERBOSE - Console text indicator for process-phase.
                | _pro_dir - Current project directory.
            :methods:
                | __init__ - Initials WriteTemplate constructor.
                | pro_dir - Property methods for set/get operations.
                | check_module - Checks project module.
                | write - Writes a template content to a project module.
    '''

    _GEN_VERBOSE: str = 'GEN_AVR8::PRO::WRITE_TEMPLATE'

    def __init__(self, verbose: bool = False) -> None:
        '''
            Initials WriteTemplate constructor.

            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :exceptions: None
        '''
        super().__init__(verbose)
        verbose_message(verbose, [f'{self._GEN_VERBOSE.lower()} init writer'])
        self._pro_dir: Optional[str] = None

    @property
    def pro_dir(self) -> Optional[str]:
        '''
            Property method for getting project dir.

            :return: Project dir | None
            :rtype: <Optional[str]>
            :exceptions: None
        '''
        return self._pro_dir

    @pro_dir.setter
    def pro_dir(self, pro_dir: Optional[str]) -> None:
        '''
            Property method for setting/creating project dir.

            :param pro_dir: Project dir | None
            :type pro_dir: <Optional[str]>
            :exceptions: ATSTypeError
        '''
        error_msg: Optional[str] = None
        error_id: Optional[int] = None
        error_msg, error_id = self.check_params([('str:pro_dir', pro_dir)])
        if error_id == self.TYPE_ERROR:
            raise ATSTypeError(error_msg)
        self._pro_dir = f'{getcwd()}/{pro_dir}'
        makedirs(f'{self._pro_dir}/build', exist_ok=True)

    def check_module(
        self, module: Optional[str], verbose: bool = False
    ) -> Optional[str]:
        '''
            Checks project module.

            :param module: Module name | None
            :type module: <Optional[str]>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :return: 'build' | 'source'| None (wrong module name).
            :rtype: <Optional[str]>
            :exceptions: ATSTypeError
        '''
        error_msg: Optional[str] = None
        error_id: Optional[int] = None
        error_msg, error_id = self.check_params([('str:module', module)])
        if error_id == self.TYPE_ERROR:
            raise ATSTypeError(error_msg)
        is_source: bool = False
        is_build: bool = False
        module_type: Optional[str] = None
        is_source = ModuleType.is_source_module(module)
        is_build = ModuleType.is_build_module(module)
        if is_source and not is_build:
            module_type = 'source'
        if not is_source and is_build:
            module_type = 'build'
        verbose_message(
            verbose, [f'{self._GEN_VERBOSE.lower()} module type', module_type]
        )
        return module_type

    def write(self, pro_data: Dict[str, str], verbose: bool = False) -> bool:
        '''
            Writes a template content to a project module.

            :param pro_data: Project data
            :type pro_data: <Dict[str, str]>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :return: True (success operation) | False
            :rtype: <bool>
            :exception: ATSTypeError
        '''
        error_msg: Optional[str] = None
        error_id: Optional[int] = None
        error_msg, error_id = self.check_params([
            ('dict:pro_data', pro_data)
        ])
        if error_id == self.TYPE_ERROR:
            raise ATSTypeError(error_msg)
        status = False
        module_type: Optional[str] = self.check_module(pro_data['module'])
        if bool(module_type):
            project: Dict[str, str] = {
                'PRO': f'{pro_data["name"]}',
                'MCU': f'{pro_data["mcu"]}',
                'OSC': f'{pro_data["osc"]}',
                'YEAR': f'{datetime.now().year}'
            }
            template = Template(pro_data['template'])
            if bool(template):
                module: Optional[str] = None
                if module_type == 'source':
                    module = f'{self._pro_dir}/{pro_data["module"]}'
                if module_type == 'build':
                    module = f'{self._pro_dir}/build/{pro_data["module"]}'
                if bool(module):
                    with open(module, 'w', encoding='utf-8') as module_file:
                        if bool(module_file):
                            verbose_message(
                                verbose,
                                [
                                    f'{self._GEN_VERBOSE.lower()} write ',
                                    module
                                ]
                            )
                            module_file.write(template.substitute(project))
                            chmod(module, 0o666)
                            file_extension: Optional[str] = None
                            if '.' in module:
                                file_extension = module.split('.')[1]
                                self.check_format(
                                    module, file_extension, verbose
                                )
                                self.check_path(module, verbose)
                                self.check_mode('w', verbose)
                                if self.is_file_ok():
                                    status = True
                            else:
                                file_extension = module
                                status = True
        return status



# -*- coding: UTF-8 -*-

'''
Module
    template_dir.py
Copyright
    Copyright (C) 2018 - 2026 Vladimir Roncevic <elektron.ronca@gmail.com>
    gen_avr8 is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    gen_avr8 is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
Info
    Defines class TemplateDir with attribute(s) and method(s).
    Creates an API for checking project template directory.
'''

import sys
from typing import List, Optional
from os.path import isdir, dirname, realpath

try:
    from ats_utilities.console_io.verbose import verbose_message
except ImportError as ats_error_message:  # pragma: no cover
    # Force exit python #######################################################
    sys.exit(f'\n{__file__}\n{ats_error_message}\n')  # pragma: no cover

__author__: str = 'Vladimir Roncevic'
__copyright__: str = '(C) 2026, https://vroncevic.github.io/gen_avr8'
__credits__: List[str] = ['Vladimir Roncevic', 'Python Software Foundation']
__license__: str = 'https://github.com/vroncevic/gen_avr8/blob/dev/LICENSE'
__version__: str = '2.6.3'
__maintainer__: str = 'Vladimir Roncevic'
__email__: str = 'elektron.ronca@gmail.com'
__status__: str = 'Updated'


class TemplateDir:
    '''
        Defines class TemplateDir with attribute(s) and method(s).
        Creates an API for checking project template directory.

        It defines:

            :attributes:
                | _GEN_VERBOSE - Console text indicator for process-phase.
                | _CONF_DIR - Configuration directory path.
                | _TEMPLATE_DIR - Template directory path.
            :methods:
                | check_dir - Checks project directory.
                | setup_conf_dir - Sets configuration directory.
                | setup_template_dir - Sets template directory.
    '''

    _GEN_VERBOSE: str = 'GEN_AVR8::PRO::_TEMPLATE_DIR'
    _CONF_DIR: str = '/../conf/'
    _TEMPLATE_DIR: str = '/../conf/template/'

    @classmethod
    def check_dir(
        cls, target_dir: Optional[str], verbose: bool = False
    ) -> bool:
        '''
            Checks project directory.

            :param target_dir: Target directory to be checked | None
            :type target_dir: <Optional[str]>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :return: True (directory ok) | False
            :rtype: <bool>
            :exceptions: None
        '''
        verbose_message(
            verbose, [f'{cls._GEN_VERBOSE.lower()} check project directory']
        )
        return isdir(f'{dirname(realpath(__file__))}{target_dir}')

    @classmethod
    def setup_conf_dir(cls, verbose: bool = False) -> Optional[str]:
        '''
            Sets configuration directory.

            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :return: Template directory | None
            :rtype: <Optional[str]>
            :exceptions: None
        '''
        conf_dir: Optional[str] = None
        if cls.check_dir(cls._CONF_DIR, verbose):
            conf_dir = f'{dirname(realpath(__file__))}{cls._CONF_DIR}'
            verbose_message(
                verbose, [f'{cls._GEN_VERBOSE.lower()} conf dir ', conf_dir]
            )
        return conf_dir

    @classmethod
    def setup_template_dir(cls, verbose: bool = False) -> Optional[str]:
        '''
            Sets template directory.

            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :return: Setup template directory | None
            :rtype: <Optional[str]>
            :exceptions: None
        '''
        template_dir: Optional[str] = None
        if cls.check_dir(cls._TEMPLATE_DIR, verbose):
            template_dir = f'{dirname(realpath(__file__))}{cls._TEMPLATE_DIR}'
            verbose_message(
                verbose, [
                    f'{cls._GEN_VERBOSE.lower()} template dir ', template_dir
                ]
            )
        return template_dir
